// Arduino sketch to control an PE4306 attenuator chips ( 0 to 31db )
// tested with attenuator board from SV1AFN ( https://www.sv1afn.com/rfattenuator.html )
// code partly by Jeff Tranter <tranter@pobox.com> 
// extended by Klaus Hirschelmann, DJ7OO  <dj7oo@t-online.de>  July 2017

// using ARDUINO Board "ProMini 3,3V" 
// supply voltage via RAW-Input DC: ~3.5-12V
// attenuator control via analogue voltage 0-3,3V connected to input #A0
// OLED via I2C-Bus using #A4 (SDA) and #A5 (SCL)
// >>>>>>> for I2C don't forget using pullup resistors <<<<<<<<<  

#include "U8glib.h"
 
int sensorValue, value;
int old_value = 100; // to make sure being different from sensored value after program start 
int LE = 2;
int CLOCK = 3;
int DATA = 4;
  
U8GLIB_SSD1306_128X64 u8g(U8G_I2C_OPT_NONE);

/////////////////////////////////////////////////////////////
void setup() 
{
  Serial.begin(9600);
  Serial.println("Start");
  pinMode(LE, OUTPUT);
  pinMode(CLOCK, OUTPUT);
  pinMode(DATA, OUTPUT);
}

/////////////////////////////////////////////////////////////
void loop() 
{
  sensorValue = analogRead(0);
  value = sensorValue/32;
  if(value>31) {value=31;}    

  if(value!=old_value)
  {
  setAttenuator(value);
  Serial.print("Attenuation; ");
  Serial.print(value);
  Serial.println("dB");
  old_value=value;
  } 
 
  delay(200);
  u8g.firstPage();  
   do {
  u8g.setFont(u8g_font_unifont);
  u8g.setPrintPos(1,10);
  u8g.print(" RF-ATT PE4306 "); 
  
  u8g.setFont(u8g_font_timB24);
  if(value<10)
  { u8g.setPrintPos(45,50); }
  else
  { u8g.setPrintPos(35,50); }
  u8g.print(value);  
  
  u8g.setFont(u8g_font_ncenB10);
  if(value<10)
  { u8g.setPrintPos(70,50); }
  else
  { u8g.setPrintPos(75,50); }
  u8g.print("dB");  
  } 
  while ( u8g.nextPage() );
}

///////////////////////////////////////////////////////////////////////
void setAttenuator(int value)
{
  int level; // Holds level of DATA line when shifting  
// Check for value range of input.
  if (value < 0 || value > 31)
    return;
// Initial state
  digitalWrite(LE, LOW);
  digitalWrite(CLOCK,LOW);  
  for (int bit = 5; bit >= 0; bit--) {
    if (bit == 0) {
      level = 0;               // LSB is always zero
    } else {
      level = ((value << 1) >> bit) & 0x01; // Level is value of bit
    }
    digitalWrite(DATA, level); // Write data value
    digitalWrite(CLOCK, HIGH); // Toggle clock high and then low
    digitalWrite(CLOCK, LOW);
  }
  digitalWrite(LE, HIGH);      // Toggle LE high to enable latch
  digitalWrite(LE, LOW);       // and then low again to hold it.
}

